@@ -1,6 +1,6 @@
 ;;; ede/cpp-root.el --- A simple way to wrap a C++ project with a single root
 
-;; Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2014 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <eric@siege-engine.com>
 
@@ -21,7 +21,7 @@
 
 ;;; Commentary:
 ;;
-;; NOTE: ede-cpp-root.el has been commented so as to also make it
+;; NOTE: ede/cpp-root.el has been commented so as to also make it
 ;;       useful for learning how to make similar project types.
 ;;
 ;; Not everyone can use automake, or an EDE project type.  For
@@ -85,7 +85,7 @@
 ;;  file name for a header in your project where most of your CPP
 ;;  macros reside.  Doing this can be easier than listing everything in
 ;;  the :spp-table option.  The files listed in :spp-files should not
-;;  start with a /, and are relative to something in :include-path.;;
+;;  start with a /, and are relative to something in :include-path.
 ;;
 ;; If you want to override the file-finding tool with your own
 ;; function you can do this:
@@ -95,7 +95,7 @@
 ;; Where FILENAME is a file in the root directory of the project.
 ;; Where MYFCN is a symbol for a function.  See:
 ;;
-;; M-x describe-class RET ede-cpp-root-project RET
+;; M-x describe-function RET ede-cpp-root-project RET
 ;;
 ;; for documentation about the locate-fcn extension.
 ;;
@@ -131,11 +131,12 @@
 ;; (add-to-list 'ede-project-class-files
 ;; 	     (ede-project-autoload "cpp-root"
 ;; 	      :name "CPP ROOT"
-;; 	      :file 'ede-cpp-root
+;; 	      :file 'ede/cpp-root
 ;; 	      :proj-file 'MY-FILE-FOR-DIR
 ;;            :proj-root 'MY-ROOT-FCN
 ;; 	      :load-type 'MY-LOAD
-;; 	      :class-sym 'ede-cpp-root)
+;; 	      :class-sym 'ede-cpp-root-project
+;;	      :safe-p t)
 ;; 	     t)
 ;;
 ;;; TODO
@@ -237,6 +238,22 @@ ROOTPROJ is nil, since there is only one project."
   ;; Snoop through our master list.
   (ede-cpp-root-file-existing dir))
 
+;;;###autoload
+(ede-add-project-autoload
+ (ede-project-autoload "cpp-root"
+		       :name "CPP ROOT"
+		       :file 'ede/cpp-root
+		       :proj-file 'ede-cpp-root-project-file-for-dir
+		       :proj-root 'ede-cpp-root-project-root
+		       :load-type 'ede-cpp-root-load
+		       :class-sym 'ede-cpp-root-project
+		       :new-p nil
+		       :safe-p t)
+ ;; When a user creates one of these, it should override any other project
+ ;; type that might happen to be in this directory, so force this to the
+ ;; very front.
+ 'unique)
+
 ;;; CLASSES
 ;;
 ;; EDE sets up projects with two kinds of objects.
@@ -255,10 +272,12 @@ ROOTPROJ is nil, since there is only one project."
 ;; level include paths, and PreProcessor macro tables.
 
 (defclass ede-cpp-root-target (ede-target)
-  ()
+  ((project :initform nil
+	    :initarg :project))
   "EDE cpp-root project target.
 All directories need at least one target.")
 
+;;;###autoload
 (defclass ede-cpp-root-project (ede-project eieio-instance-tracker)
   ((tracking-symbol :initform 'ede-cpp-root-project-list)
    (include-path :initarg :include-path
@@ -322,6 +341,15 @@ The function symbol must take two arguments:
 It should return the fully qualified file name passed in from NAME.  If that file does not
 exist, it should return nil."
 	       )
+   (compile-command :initarg :compile-command
+		    :initform nil
+		    :type (or null string function)
+		    :documentation
+		    "Compilation command that will be used for this project.
+It could be string or function that will accept proj argument and should return string.
+The string will be passed to 'compile' function that will be issued in root
+directory of project."
+		    )
    )
   "EDE cpp-root project class.
 Each directory needs a project file to control it.")
@@ -349,7 +377,7 @@ Each directory needs a project file to control it.")
     (when (or (not (file-exists-p f))
 	      (file-directory-p f))
       (delete-instance this)
-      (error ":file for ede-cpp-root must be a file"))
+      (error ":file for ede-cpp-root-project must be a file"))
     (oset this :file f)
     (oset this :directory (file-name-directory f))
     (ede-project-directory-remove-hash (file-name-directory f))
@@ -387,7 +415,8 @@ If one doesn't exist, create a new one for this directory."
                  :name (file-name-nondirectory
 			(directory-file-name dir))
 		 :path dir
-		 :source nil))
+		 :source nil
+		 :project proj))
       (object-add-to-list proj :targets ans)
       )
     ans))
@@ -427,6 +456,7 @@ This knows details about or source tree."
 	    ;; Else, do the usual.
 	    (setq ans (call-next-method)))
 	  )))
+    ;; TODO - does this call-next-method happen twice.  Is that bad??  Why is it here?
     (or ans (call-next-method))))
 
 (defmethod ede-project-root ((this ede-cpp-root-project))
@@ -463,15 +493,6 @@ This is for project include paths and spp source files."
 
     filename))
 
-(defmethod ede-set-project-variables ((project ede-cpp-root-project) &optional buffer)
-  "Set variables local to PROJECT in BUFFER.
-Also set up the lexical preprocessor map."
-  (call-next-method)
-  (when (and (featurep 'semantic/c) (featurep 'semantic/lex-spp))
-    (setq semantic-lex-spp-project-macro-symbol-obarray
-	  (semantic-lex-make-spp-table (oref project spp-table)))
-    ))
-
 (defmethod ede-system-include-path ((this ede-cpp-root-project))
   "Get the system include path used by project THIS."
   (oref this system-include-path))
@@ -486,24 +507,72 @@ Also set up the lexical preprocessor map."
      (lambda (F)
        (let* ((expfile (ede-expand-filename root F))
 	      (table (when expfile
-		       (semanticdb-file-table-object expfile)))
+		       ;; Disable EDE init on preprocessor file load
+		       ;; otherwise we recurse, cause errs, etc.
+		       (let ((ede-constructing t))
+			 (semanticdb-file-table-object expfile))))
 	      )
-	 (when (not table)
+	 (cond
+	  ((not (file-exists-p expfile))
 	   (message "Cannot find file %s in project." F))
-	 (when (and table (semanticdb-needs-refresh-p table))
-	   (semanticdb-refresh-table table)
-	   (setq spp (append spp (oref table lexical-table))))))
+	  ((string= expfile (buffer-file-name))
+	   ;; Don't include this file in it's own spp table.
+	   )
+	  ((not table)
+	   (message "No db table available for %s." expfile))
+	  (t
+	   (when (semanticdb-needs-refresh-p table)
+	     (semanticdb-refresh-table table))
+	   (setq spp (append spp (oref table lexical-table)))))))
      (oref this spp-files))
     spp))
 
 (defmethod ede-system-include-path ((this ede-cpp-root-target))
-  "Get the system include path used by project THIS."
+  "Get the system include path used by target THIS."
   (ede-system-include-path (ede-target-parent this)))
 
 (defmethod ede-preprocessor-map ((this ede-cpp-root-target))
   "Get the pre-processor map for project THIS."
   (ede-preprocessor-map  (ede-target-parent this)))
 
+(defmethod project-compile-project ((proj ede-cpp-root-project) &optional command)
+  "Compile the entire current project PROJ.
+Argument COMMAND is the command to use when compiling."
+  ;; we need to be in the proj root dir for this to work
+  (let* ((cmd (oref proj :compile-command))
+	 (ov (oref proj :local-variables))
+	 (lcmd (when ov (cdr (assoc 'compile-command ov))))
+	 (cmd-str (cond
+		   ((stringp cmd) cmd)
+		   ((functionp cmd) (funcall cmd proj))
+		   ((stringp lcmd) lcmd)
+		   ((functionp lcmd) (funcall lcmd proj)))))
+    (when cmd-str
+	(let ((default-directory (ede-project-root-directory proj)))
+	(compile cmd-str)))))
+
+(defmethod project-compile-target ((obj ede-cpp-root-target) &optional command)
+  "Compile the current target OBJ.
+Argument COMMAND is the command to use for compiling the target."
+  (when (oref obj :project)
+    (project-compile-project (oref obj :project) command)))
+
+
+;;; Quick Hack
+(defun ede-create-lots-of-projects-under-dir (dir projfile &rest attributes)
+  "Create a bunch of projects under directory DIR.
+PROJFILE is a file name sans directory that indicates a subdirectory
+is a project directory.
+Generic ATTRIBUTES, such as :include-path can be added.
+Note: This needs some work."
+  (let ((files (directory-files dir t)))
+    (dolist (F files)
+      (if (file-exists-p (expand-file-name projfile F))
+	  `(ede-cpp-root-project (file-name-nondirectory F)
+				 :name (file-name-nondirectory F)
+				 :file (expand-file-name projfile F)
+				 attributes)))))
+
 (provide 'ede/cpp-root)
 
 ;; Local variables:
@@ -511,5 +580,4 @@ Also set up the lexical preprocessor map."
 ;; generated-autoload-load-name: "ede/cpp-root"
 ;; End:
 
-;; arch-tag: c3ac8160-cba6-447e-8b9c-accb7e2d942e
 ;;; ede/cpp-root.el ends here
