@@ -1,7 +1,6 @@
 ;;; semantic/sb.el --- Semantic tag display for speedbar
 
-;; Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-;;   2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+;; Copyright (C) 1999-2014 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Keywords: syntax
@@ -44,6 +43,11 @@ This will replace the named bucket that would have usually occurred here."
   :group 'speedbar
   :type 'integer)
 
+(defvar semantic-sb-filter-tags-of-class '(code)
+  "Tags classes to not display in speedbar.
+Make this buffer local for modes that have different types of tags
+that should be ignored.")
+
 (defcustom semantic-sb-button-format-tag-function 'semantic-format-tag-abbreviate
   "*Function called to create the text for a but from a token."
   :group 'speedbar
@@ -88,7 +92,7 @@ Restore the old current buffer when completed."
 ;;    @ type
 ;;    = default value
 ;;
-;;  +> keywrd Type
+;;  +> keyword Type
 ;;   +> type part
 ;;
 ;;  +>  -> click to see additional information
@@ -314,12 +318,12 @@ TEXT TOKEN and INDENT are the details."
     (save-excursion (speedbar-stealthy-updates))
     (semantic-go-to-tag token parent)
     (switch-to-buffer (current-buffer))
-    ;; Reset the timer with a new timeout when cliking a file
+    ;; Reset the timer with a new timeout when clicking a file
     ;; in case the user was navigating directories, we can cancel
     ;; that other timer.
     ;; (speedbar-set-timer dframe-update-speed)
     ;;(recenter)
-    (speedbar-maybee-jump-to-attached-frame)
+    (dframe-maybee-jump-to-attached-frame)
     (run-hooks 'speedbar-visiting-tag-hook)))
 
 (defun semantic-sb-expand-group (text token indent)
@@ -400,13 +404,18 @@ Returns the tag list, or t for an error."
     (if (listp out)
 	(condition-case nil
 	    (progn
-	      ;; This brings externally defind methods into
+	      ;; This brings externally defined methods into
 	      ;; their classes, and creates meta classes for
 	      ;; orphans.
 	      (setq out (semantic-adopt-external-members out))
 	      ;; Dump all the tokens into buckets.
 	      (semantic-sb-with-tag-buffer (car out)
-		(semantic-bucketize out)))
+		(semantic-bucketize out nil
+				    (lambda (tagsin)
+				      ;; Remove all boring tags.
+				      (semantic-filter-tags-by-class
+				       semantic-sb-filter-tags-of-class
+				       tagsin)))))
 	  (error t))
       t)))
 
@@ -416,5 +425,8 @@ Returns the tag list, or t for an error."
 
 (provide 'semantic/sb)
 
-;; arch-tag: 82aa0570-9e27-41a3-a834-2641dbb2f829
+;; Local variables:
+;; generated-autoload-load-name: "semantic/sb"
+;; End:
+
 ;;; semantic/sb.el ends here
