@@ -1,5 +1,6 @@
-;;; epa.el --- the EasyPG Assistant
-;; Copyright (C) 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;;; epa.el --- the EasyPG Assistant -*- lexical-binding: t -*-
+
+;; Copyright (C) 2006-2014 Free Software Foundation, Inc.
 
 ;; Author: Daiki Ueno <ueno@unixuser.org>
 ;; Keywords: PGP, GnuPG
@@ -30,11 +31,11 @@
 (defgroup epa nil
   "The EasyPG Assistant"
   :version "23.1"
+  :link '(custom-manual "(epa) Top")
   :group 'epg)
 
 (defcustom epa-popup-info-window t
-  "If non-nil, status information from epa commands is displayed on
-the separate window."
+  "If non-nil, display status information from epa commands in another window."
   :type 'boolean
   :group 'epa)
 
@@ -48,98 +49,67 @@ the separate window."
   :version "23.1"
   :group 'epa)
 
+(defcustom epa-mail-aliases nil
+  "Alist of aliases of email addresses that stand for encryption keys.
+Each element is a list of email addresses (ALIAS EXPANSIONS...).
+When one of the recipients of a message being encrypted is ALIAS,
+instead of encrypting it for ALIAS, encrypt it for EXPANSIONS...
+
+If EXPANSIONS is empty, ignore ALIAS as regards encryption.
+This is a handy way to avoid warnings about addresses that you don't
+have any key for.
+
+The command `epa-mail-encrypt' uses this."
+  :type '(repeat (cons (string :tag "Alias") (repeat (string :tag "Expansion"))))
+  :group 'epa
+  :version "24.4")
+
 (defface epa-validity-high
-  `((((class color) (background dark))
-     (:foreground "PaleTurquoise"
-		  ,@(if (assq ':weight custom-face-attributes)
-			'(:weight bold)
-		      '(:bold t))))
-    (t
-     (,@(if (assq ':weight custom-face-attributes)
-	    '(:weight bold)
-	  '(:bold t)))))
-  "Face used for displaying the high validity."
+  '((default :weight bold)
+    (((class color) (background dark)) :foreground "PaleTurquoise"))
+  "Face for high validity EPA information."
   :group 'epa-faces)
 
 (defface epa-validity-medium
-  `((((class color) (background dark))
-     (:foreground "PaleTurquoise"
-		  ,@(if (assq ':slant custom-face-attributes)
-			'(:slant italic)
-		      '(:italic t))))
-    (t
-     (,@(if (assq ':slant custom-face-attributes)
-	    '(:slant italic)
-	  '(:italic t)))))
-  "Face used for displaying the medium validity."
+  '((default :slant italic)
+    (((class color) (background dark)) :foreground "PaleTurquoise"))
+  "Face for medium validity EPA information."
   :group 'epa-faces)
 
 (defface epa-validity-low
-  `((t
-     (,@(if (assq ':slant custom-face-attributes)
-	    '(:slant italic)
-	  '(:italic t)))))
+  '((t :slant italic))
   "Face used for displaying the low validity."
   :group 'epa-faces)
 
 (defface epa-validity-disabled
-  `((t
-     (,@(if (assq ':slant custom-face-attributes)
-	    '(:slant italic)
-	  '(:italic t))
-	:inverse-video t)))
+  '((t :slant italic :inverse-video t))
   "Face used for displaying the disabled validity."
   :group 'epa-faces)
 
 (defface epa-string
   '((((class color) (background dark))
-     (:foreground "lightyellow"))
+     :foreground "lightyellow")
     (((class color) (background light))
-     (:foreground "blue4")))
+     :foreground "blue4"))
   "Face used for displaying the string."
   :group 'epa-faces)
 
 (defface epa-mark
-  `((((class color) (background dark))
-     (:foreground "orange"
-		  ,@(if (assq ':weight custom-face-attributes)
-			'(:weight bold)
-		      '(:bold t))))
-    (((class color) (background light))
-     (:foreground "red"
-		  ,@(if (assq ':weight custom-face-attributes)
-			'(:weight bold)
-		      '(:bold t))))
-    (t
-     (,@(if (assq ':weight custom-face-attributes)
-	    '(:weight bold)
-	  '(:bold t)))))
+  '((default :weight bold)
+    (((class color) (background dark))  :foreground "orange")
+    (((class color) (background light)) :foreground "red"))
   "Face used for displaying the high validity."
   :group 'epa-faces)
 
 (defface epa-field-name
-  `((((class color) (background dark))
-     (:foreground "PaleTurquoise"
-		  ,@(if (assq ':weight custom-face-attributes)
-			'(:weight bold)
-		      '(:bold t))))
-    (t
-     (,@(if (assq ':weight custom-face-attributes)
-	    '(:weight bold)
-	  '(:bold t)))))
+  '((default :weight bold)
+    (((class color) (background dark)) :foreground "PaleTurquoise"))
   "Face for the name of the attribute field."
   :group 'epa)
 
 (defface epa-field-body
-  `((((class color) (background dark))
-     (:foreground "turquoise"
-		  ,@(if (assq ':slant custom-face-attributes)
-			'(:slant italic)
-		      '(:italic t))))
-    (t
-     (,@(if (assq ':slant custom-face-attributes)
-	    '(:slant italic)
-	  '(:italic t)))))
+  '((default :slant italic)
+    (((class color) (background dark)) :foreground "turquoise"))
   "Face for the body of the attribute field."
   :group 'epa)
 
@@ -176,18 +146,18 @@ the separate window."
     (20 . ?G)))
 
 (defvar epa-protocol 'OpenPGP
-  "*The default protocol.
+  "The default protocol.
 The value can be either OpenPGP or CMS.
 
 You should bind this variable with `let', but do not set it globally.")
 
 (defvar epa-armor nil
-  "*If non-nil, epa commands create ASCII armored output.
+  "If non-nil, epa commands create ASCII armored output.
 
 You should bind this variable with `let', but do not set it globally.")
 
 (defvar epa-textmode nil
-  "*If non-nil, epa commands treat input files as text.
+  "If non-nil, epa commands treat input files as text.
 
 You should bind this variable with `let', but do not set it globally.")
 
@@ -213,8 +183,9 @@ You should bind this variable with `let', but do not set it globally.")
     (define-key keymap "g" 'revert-buffer)
     (define-key keymap "n" 'next-line)
     (define-key keymap "p" 'previous-line)
-    (define-key keymap " " 'scroll-up)
-    (define-key keymap [delete] 'scroll-down)
+    (define-key keymap " " 'scroll-up-command)
+    (define-key keymap [?\S-\ ] 'scroll-down-command)
+    (define-key keymap [delete] 'scroll-down-command)
     (define-key keymap "q" 'epa-exit-buffer)
     (define-key keymap [menu-bar epa-key-list-mode] (cons "Keys" menu-map))
     (define-key menu-map [epa-key-list-unmark-key]
@@ -234,11 +205,11 @@ You should bind this variable with `let', but do not set it globally.")
       '(menu-item "Decrypt File..." epa-decrypt-file
 		  :help "Decrypt FILE"))
     (define-key menu-map [epa-encrypt-file]
-      '(menu-item "Encrypt File.." epa-encrypt-file
+      '(menu-item "Encrypt File..." epa-encrypt-file
 		  :help "Encrypt FILE for RECIPIENTS"))
     (define-key menu-map [separator-epa-key-list] '(menu-item "--"))
     (define-key menu-map [epa-key-list-delete-keys]
-      '(menu-item "Delete keys" epa-delete-keys
+      '(menu-item "Delete Keys" epa-delete-keys
 		  :help "Delete Marked Keys"))
     (define-key menu-map [epa-key-list-import-keys]
       '(menu-item "Import Keys" epa-import-keys
@@ -268,7 +239,7 @@ You should bind this variable with `let', but do not set it globally.")
   :action 'epa--key-widget-action
   :help-echo 'epa--key-widget-help-echo)
 
-(defun epa--key-widget-action (widget &optional event)
+(defun epa--key-widget-action (widget &optional _event)
   (save-selected-window
     (epa--show-key (widget-get widget :value))))
 
@@ -301,62 +272,40 @@ You should bind this variable with `let', but do not set it globally.")
 	  (epg-sub-key-id (car (epg-key-sub-key-list
 				(widget-get widget :value))))))
 
-(eval-and-compile
-  (if (fboundp 'encode-coding-string)
-      (defalias 'epa--encode-coding-string 'encode-coding-string)
-    (defalias 'epa--encode-coding-string 'identity)))
+(defalias 'epa--encode-coding-string
+  (if (fboundp 'encode-coding-string) #'encode-coding-string #'identity))
 
-(eval-and-compile
-  (if (fboundp 'decode-coding-string)
-      (defalias 'epa--decode-coding-string 'decode-coding-string)
-    (defalias 'epa--decode-coding-string 'identity)))
+(defalias 'epa--decode-coding-string
+  (if (fboundp 'decode-coding-string) #'decode-coding-string #'identity))
 
-(defun epa-key-list-mode ()
+(define-derived-mode epa-key-list-mode special-mode "Keys"
   "Major mode for `epa-list-keys'."
-  (kill-all-local-variables)
   (buffer-disable-undo)
-  (setq major-mode 'epa-key-list-mode
-	mode-name "Keys"
-	truncate-lines t
+  (setq truncate-lines t
 	buffer-read-only t)
-  (use-local-map epa-key-list-mode-map)
-  (make-local-variable 'font-lock-defaults)
-  (setq font-lock-defaults '(epa-font-lock-keywords t))
+  (setq-local font-lock-defaults '(epa-font-lock-keywords t))
   ;; In XEmacs, auto-initialization of font-lock is not effective
   ;; if buffer-file-name is not set.
   (font-lock-set-defaults)
   (make-local-variable 'epa-exit-buffer-function)
-  (make-local-variable 'revert-buffer-function)
-  (setq revert-buffer-function 'epa--key-list-revert-buffer)
-  (run-mode-hooks 'epa-key-list-mode-hook))
+  (setq-local revert-buffer-function #'epa--key-list-revert-buffer))
 
-(defun epa-key-mode ()
+(define-derived-mode epa-key-mode special-mode "Key"
   "Major mode for a key description."
-  (kill-all-local-variables)
   (buffer-disable-undo)
-  (setq major-mode 'epa-key-mode
-	mode-name "Key"
-	truncate-lines t
+  (setq truncate-lines t
 	buffer-read-only t)
-  (use-local-map epa-key-mode-map)
-  (make-local-variable 'font-lock-defaults)
-  (setq font-lock-defaults '(epa-font-lock-keywords t))
+  (setq-local font-lock-defaults '(epa-font-lock-keywords t))
   ;; In XEmacs, auto-initialization of font-lock is not effective
   ;; if buffer-file-name is not set.
   (font-lock-set-defaults)
-  (make-local-variable 'epa-exit-buffer-function)
-  (run-mode-hooks 'epa-key-mode-hook))
+  (make-local-variable 'epa-exit-buffer-function))
 
-(defun epa-info-mode ()
+(define-derived-mode epa-info-mode special-mode "Info"
   "Major mode for `epa-info-buffer'."
-  (kill-all-local-variables)
   (buffer-disable-undo)
-  (setq major-mode 'epa-info-mode
-	mode-name "Info"
-	truncate-lines t
-	buffer-read-only t)
-  (use-local-map epa-info-mode-map)
-  (run-mode-hooks 'epa-info-mode-hook))
+  (setq truncate-lines t
+	buffer-read-only t))
 
 (defun epa-mark-key (&optional arg)
   "Mark a key on the current line.
@@ -459,12 +408,11 @@ If ARG is non-nil, mark the key."
      (list nil)))
   (epa--list-keys name t))
 
-(defun epa--key-list-revert-buffer (&optional ignore-auto noconfirm)
+(defun epa--key-list-revert-buffer (&optional _ignore-auto _noconfirm)
   (apply #'epa--list-keys epa-list-keys-arguments))
 
 (defun epa--marked-keys ()
-  (or (save-excursion
-	(set-buffer epa-keys-buffer)
+  (or (with-current-buffer epa-keys-buffer
 	(goto-char (point-min))
 	(let (keys key)
 	  (while (re-search-forward "^\\*" nil t)
@@ -472,19 +420,18 @@ If ARG is non-nil, mark the key."
 					     'epa-key))
 		(setq keys (cons key keys))))
 	  (nreverse keys)))
-      (save-excursion
-	(beginning-of-line)
-	(let ((key (get-text-property (point) 'epa-key)))
-	  (if key
-	      (list key))))))
+      (let ((key (get-text-property (point-at-bol) 'epa-key)))
+	(if key
+	    (list key)))))
 
 (defun epa--select-keys (prompt keys)
-  (save-excursion
-    (unless (and epa-keys-buffer
-		 (buffer-live-p epa-keys-buffer))
-      (setq epa-keys-buffer (generate-new-buffer "*Keys*")))
-    (set-buffer epa-keys-buffer)
+  (unless (and epa-keys-buffer
+               (buffer-live-p epa-keys-buffer))
+    (setq epa-keys-buffer (generate-new-buffer "*Keys*")))
+  (with-current-buffer epa-keys-buffer
     (epa-key-list-mode)
+    ;; C-c C-c is the usual way to finish the selection (bug#11159).
+    (define-key (current-local-map) "\C-c\C-c" 'exit-recursive-edit)
     (let ((inhibit-read-only t)
 	  buffer-read-only)
       (erase-buffer)
@@ -493,13 +440,13 @@ If ARG is non-nil, mark the key."
 - `\\[epa-mark-key]' to mark a key on the line
 - `\\[epa-unmark-key]' to unmark a key on the line\n"))
       (widget-create 'link
-		     :notify (lambda (&rest ignore) (abort-recursive-edit))
+		     :notify (lambda (&rest _ignore) (abort-recursive-edit))
 		     :help-echo
 		     (substitute-command-keys
 		      "Click here or \\[abort-recursive-edit] to cancel")
 		     "Cancel")
       (widget-create 'link
-		     :notify (lambda (&rest ignore) (exit-recursive-edit))
+		     :notify (lambda (&rest _ignore) (exit-recursive-edit))
 		     :help-echo
 		     (substitute-command-keys
 		      "Click here or \\[exit-recursive-edit] to finish")
@@ -510,13 +457,12 @@ If ARG is non-nil, mark the key."
       (set-keymap-parent (current-local-map) widget-keymap)
       (setq epa-exit-buffer-function #'abort-recursive-edit)
       (goto-char (point-min))
-      (pop-to-buffer (current-buffer)))
+      (let ((display-buffer-mark-dedicated 'soft))
+        (pop-to-buffer (current-buffer))))
     (unwind-protect
 	(progn
 	  (recursive-edit)
 	  (epa--marked-keys))
-      (if (get-buffer-window epa-keys-buffer)
-	  (delete-window (get-buffer-window epa-keys-buffer)))
       (kill-buffer epa-keys-buffer))))
 
 ;;;###autoload
@@ -615,8 +561,7 @@ If SECRET is non-nil, list secret keys instead of public keys."
 	  (setq epa-info-buffer (generate-new-buffer "*Info*")))
 	(if (get-buffer-window epa-info-buffer)
 	    (delete-window (get-buffer-window epa-info-buffer)))
-	(save-excursion
-	  (set-buffer epa-info-buffer)
+	(with-current-buffer epa-info-buffer
 	  (let ((inhibit-read-only t)
 		buffer-read-only)
 	    (erase-buffer)
@@ -634,13 +579,18 @@ If SECRET is non-nil, list secret keys instead of public keys."
     (message "%s" info)))
 
 (defun epa-display-verify-result (verify-result)
+  (declare (obsolete epa-display-info "23.1"))
   (epa-display-info (epg-verify-result-to-string verify-result)))
-(make-obsolete 'epa-display-verify-result 'epa-display-info)
 
 (defun epa-passphrase-callback-function (context key-id handback)
   (if (eq key-id 'SYM)
-      (read-passwd "Passphrase for symmetric encryption: "
-		   (eq (epg-context-operation context) 'encrypt))
+      (read-passwd
+       (format "Passphrase for symmetric encryption%s: "
+	       ;; Add the file name to the prompt, if any.
+	       (if (stringp handback)
+		   (format " for %s" handback)
+		 ""))
+       (eq (epg-context-operation context) 'encrypt))
     (read-passwd
      (if (eq key-id 'PIN)
 	"Passphrase for PIN: "
@@ -649,38 +599,51 @@ If SECRET is non-nil, list secret keys instead of public keys."
 	     (format "Passphrase for %s %s: " key-id (cdr entry))
 	   (format "Passphrase for %s: " key-id)))))))
 
-(defun epa-progress-callback-function (context what char current total
+(defun epa-progress-callback-function (_context what _char current total
 					       handback)
-  (message "%s%d%% (%d/%d)" (or handback
-				(concat what ": "))
-	   (if (> total 0) (floor (* (/ current (float total)) 100)) 0)
-	   current total))
+  (let ((prompt (or handback
+		    (format "Processing %s: " what))))
+    ;; According to gnupg/doc/DETAIL: a "total" of 0 indicates that
+    ;; the total amount is not known. The condition TOTAL && CUR ==
+    ;; TOTAL may be used to detect the end of an operation.
+    (if (> total 0)
+	(if (= current total)
+	    (message "%s...done" prompt)
+	  (message "%s...%d%%" prompt
+		   (floor (* (/ current (float total)) 100))))
+      (message "%s..." prompt))))
+
+(defun epa-read-file-name (input)
+  "Interactively read an output file name based on INPUT file name."
+  (setq input (file-name-sans-extension (expand-file-name input)))
+  (expand-file-name
+   (read-file-name
+    (concat "To file (default " (file-name-nondirectory input) ") ")
+    (file-name-directory input)
+    input)))
 
 ;;;###autoload
-(defun epa-decrypt-file (file)
-  "Decrypt FILE."
-  (interactive "fFile: ")
-  (setq file (expand-file-name file))
-  (let* ((default-name (file-name-sans-extension file))
-	 (plain (expand-file-name
-		 (read-file-name
-		  (concat "To file (default "
-			  (file-name-nondirectory default-name)
-			  ") ")
-		  (file-name-directory default-name)
-		  default-name)))
-	 (context (epg-make-context epa-protocol)))
+(defun epa-decrypt-file (decrypt-file &optional plain-file)
+  "Decrypt DECRYPT-FILE into PLAIN-FILE.
+If you do not specify PLAIN-FILE, this functions prompts for the value to use."
+  (interactive
+   (let* ((file (read-file-name "File to decrypt: "))
+	  (plain (epa-read-file-name file)))
+     (list file plain)))
+  (or plain-file (setq plain-file (epa-read-file-name decrypt-file)))
+  (setq decrypt-file (expand-file-name decrypt-file))
+  (let ((context (epg-make-context epa-protocol)))
     (epg-context-set-passphrase-callback context
 					 #'epa-passphrase-callback-function)
     (epg-context-set-progress-callback context
 				       (cons
 					#'epa-progress-callback-function
 					(format "Decrypting %s..."
-						(file-name-nondirectory file))))
-    (message "Decrypting %s..." (file-name-nondirectory file))
-    (epg-decrypt-file context file plain)
-    (message "Decrypting %s...wrote %s" (file-name-nondirectory file)
-	     (file-name-nondirectory plain))
+						(file-name-nondirectory decrypt-file))))
+    (message "Decrypting %s..." (file-name-nondirectory decrypt-file))
+    (epg-decrypt-file context decrypt-file plain-file)
+    (message "Decrypting %s...wrote %s" (file-name-nondirectory decrypt-file)
+	     (file-name-nondirectory plain-file))
     (if (epg-context-result-for context 'verify)
 	(epa-display-info (epg-verify-result-to-string
 			   (epg-context-result-for context 'verify))))))
@@ -716,8 +679,7 @@ If SECRET is non-nil, list secret keys instead of public keys."
 	     (setq type 'detached))
 	    ((eq c ??)
 	     (with-output-to-temp-buffer "*Help*"
-	       (save-excursion
-		 (set-buffer standard-output)
+	       (with-current-buffer standard-output
 		 (insert "\
 n - Create a normal signature
 c - Create a cleartext signature
@@ -797,10 +759,15 @@ If no one is selected, symmetric encryption will be performed.  ")))
 	     (file-name-nondirectory cipher))))
 
 ;;;###autoload
-(defun epa-decrypt-region (start end)
+(defun epa-decrypt-region (start end &optional make-buffer-function)
   "Decrypt the current region between START and END.
 
-Don't use this command in Lisp programs!
+If MAKE-BUFFER-FUNCTION is non-nil, call it to prepare an output buffer.
+It should return that buffer.  If it copies the input, it should
+delete the text now being decrypted.  It should leave point at the
+proper place to insert the plaintext.
+
+Be careful about using this command in Lisp programs!
 Since this function operates on regions, it does some tricks such
 as coding-system detection and unibyte/multibyte conversion.  If
 you are sure how the data in the region should be treated, you
@@ -832,16 +799,19 @@ For example:
 		   (or coding-system-for-read
 		       (get-text-property start 'epa-coding-system-used)
 		       'undecided)))
-      (if (y-or-n-p "Replace the original text? ")
-	  (let ((inhibit-read-only t)
-		buffer-read-only)
-	    (delete-region start end)
-	    (goto-char start)
-	    (insert plain))
-	(with-output-to-temp-buffer "*Temp*"
-	  (set-buffer standard-output)
-	  (insert plain)
-	  (epa-info-mode)))
+      (if make-buffer-function
+	  (with-current-buffer (funcall make-buffer-function)
+	    (let ((inhibit-read-only t))
+	      (insert plain)))
+	(if (y-or-n-p "Replace the original text? ")
+	    (let ((inhibit-read-only t))
+	      (delete-region start end)
+	      (goto-char start)
+	      (insert plain))
+	  (with-output-to-temp-buffer "*Temp*"
+	    (set-buffer standard-output)
+	      (insert plain)
+	      (epa-info-mode))))
       (if (epg-context-result-for context 'verify)
 	  (epa-display-info (epg-verify-result-to-string
 			     (epg-context-result-for context 'verify)))))))
@@ -850,12 +820,13 @@ For example:
   (if (featurep 'xemacs)
       (if (fboundp 'find-coding-system)
 	  (find-coding-system mime-charset))
+    ;; Find the first coding system which corresponds to MIME-CHARSET.
     (let ((pointer (coding-system-list)))
       (while (and pointer
-		  (eq (coding-system-get (car pointer) 'mime-charset)
-		      mime-charset))
+		  (not (eq (coding-system-get (car pointer) 'mime-charset)
+			   mime-charset)))
 	(setq pointer (cdr pointer)))
-      pointer)))
+      (car pointer))))
 
 ;;;###autoload
 (defun epa-decrypt-armor-in-region (start end)
@@ -874,7 +845,7 @@ See the reason described in the `epa-decrypt-region' documentation."
 		armor-end (re-search-forward "^-----END PGP MESSAGE-----$"
 					     nil t))
 	  (unless armor-end
-	    (error "No armor tail"))
+	    (error "Encryption armor beginning has no matching end"))
 	  (goto-char armor-start)
 	  (let ((coding-system-for-read
 		 (or coding-system-for-read
@@ -962,10 +933,10 @@ See the reason described in the `epa-verify-region' documentation."
 	    (error "No cleartext tail"))
 	  (epa-verify-region cleartext-start cleartext-end))))))
 
-(eval-and-compile
+(defalias 'epa--select-safe-coding-system
   (if (fboundp 'select-safe-coding-system)
-      (defalias 'epa--select-safe-coding-system 'select-safe-coding-system)
-    (defun epa--select-safe-coding-system (from to)
+      #'select-safe-coding-system
+    (lambda (_from _to)
       buffer-file-coding-system)))
 
 ;;;###autoload
@@ -1037,16 +1008,16 @@ If no one is selected, default secret key is used.  "
 				 'start-open t
 				 'end-open t)))))
 
-(eval-and-compile
+(defalias 'epa--derived-mode-p
   (if (fboundp 'derived-mode-p)
-      (defalias 'epa--derived-mode-p 'derived-mode-p)
-    (defun epa--derived-mode-p (&rest modes)
+      #'derived-mode-p
+    (lambda (&rest modes)
       "Non-nil if the current major mode is derived from one of MODES.
 Uses the `derived-mode-parent' property of the symbol to trace backwards."
       (let ((parent major-mode))
-	(while (and (not (memq parent modes))
-		    (setq parent (get parent 'derived-mode-parent))))
-	parent))))
+        (while (and (not (memq parent modes))
+                    (setq parent (get parent 'derived-mode-parent))))
+        parent))))
 
 ;;;###autoload
 (defun epa-encrypt-region (start end recipients sign signers)
@@ -1149,6 +1120,7 @@ If no one is selected, symmetric encryption will be performed.  ")
     (if (epg-context-result-for context 'import)
 	(epa-display-info (epg-import-result-to-string
 			   (epg-context-result-for context 'import))))
+    ;; FIXME: Why not use the (otherwise unused) epa--derived-mode-p?
     (if (eq major-mode 'epa-key-list-mode)
 	(apply #'epa--list-keys epa-list-keys-arguments))))
 
@@ -1221,7 +1193,8 @@ between START and END."
   "Insert selected KEYS after the point."
   (interactive
    (list (epa-select-keys (epg-make-context epa-protocol)
-			  "Select keys to export.  ")))
+				"Select keys to export.
+If no one is selected, default public key is exported.  ")))
   (let ((context (epg-make-context epa-protocol)))
     ;;(epg-context-set-armor context epa-armor)
     (epg-context-set-armor context t)
@@ -1251,5 +1224,4 @@ between START and END."
 
 (provide 'epa)
 
-;; arch-tag: 38d20ced-20d5-4137-b17a-f206335423d7
 ;;; epa.el ends here
