@@ -5,6 +5,55 @@ setlock -n /tmp/getmail.lock && echo getmail isn\'t running
 
 # adsgsdg
 
+if foo; then
+    if bar; then
+	toto
+    fi
+fi                              # bug#15613
+
+case $X in
+    foo)
+        do_something
+        ;;
+    arg=*)			# bug#12953
+        do_something_else_based_on_arg
+        ;;
+    *)
+        default
+        ;;
+esac
+
+{				# bug#17621
+    foo1 &&
+	foo2 &&
+        bar
+
+    foo1 &&     \
+        foo2 && \
+        bar
+}
+
+for foo in bar; do              #  bug#17721
+    [ -e $foo ] && {
+        echo t
+    } && {
+	echo r
+    }
+done
+
+for foo in bar; do              # bug#17896
+    [ -e $foo ] && [ -e $bar ] && {
+        echo just fine thanks
+    }
+done
+
+filter_3 ()                     # bug#17842
+{
+    tr -d '"`' | tr '	' ' ' | \
+        awk -F\; -f filter.awk | \
+	grep -v "^," | sort -t, -k2,2
+}
+
 echo -n $(( 5 << 2 ))
 # This should not be treated as a heredoc (bug#12770).
 2
@@ -17,7 +66,7 @@ echo $[1<<8]                    # bug#11263
 declare -a VERSION
 for i in $(ls "$PREFIX/sbin") ; do
     echo -e $N')' $i
-    VERSION[${#VERSION[*]}]=$i         #bug#11946.
+    VERSION[${#VERSION[*]}]=$i  # bug#11946.
     N=$(($N + 1))
 done
 
